@@ -16,6 +16,7 @@
 from abc import abstractmethod
 import numpy as np
 from datetime import datetime
+import logging
 
 
 class StoppingCriteria:
@@ -24,10 +25,17 @@ class StoppingCriteria:
         """
         Parent class for the stopping criteria
         """
+        self.is_converged = False
+        self.logger = logging.getLogger("oc_logger")
         self.max_eval = stopping_criteria.setdefault("max_eval", 10**10)
         self.time_lim = stopping_criteria.setdefault("time_lim", 10**10)
         self.direct_search_start_time = datetime.now()
-        pass
+        self.change_based_stop = stopping_criteria.setdefault("change_based_stop", {"cbs_funct_evals": 1,
+                                                                                    "cbs_change": 0})
+        # if "cbs_funct_evals" not in self.change_based_stop:
+        #     self.change_based_stop["cbs_funct_evals"] = 1
+        self.curr_FoM_track: list = []
+
 
     @abstractmethod
     def check_stopping_criteria(self, **kwargs) -> None:
@@ -55,46 +63,6 @@ class StoppingCriteria:
             is_converged = True
         return [is_converged, terminate_reason]
 
-    # def check_func_eval_total(self, func_evaluations_single_direct_search: int) -> [bool, str]:
-    #     """
-    #     Check whether the total maximum number of function evaluations has been exceeded
-    #
-    #     :param float func_evaluations_single_direct_search: number of current function evaluation
-    #     :return bool is_converged: True if the stopping criterion is fulfilled
-    #     :return str terminate_reason: reason for the terminatiom
-    #     """
-    #
-    #     # ToDo: This is extremely complicated and ther has to ba a simple way to do this properly.
-    #     #  Also this might break for very low max_eval and max_eval_total
-    #     #  AND this actually breaks if we continue after reaching max_eval_total !!!
-    #     #  But we don't reach that because we stop the optimization anyway
-    #
-    #     if self.max_eval_total is not None:
-    #
-    #         local_func_eval_track = 0
-    #
-    #         if self.temporary_func_eval_track > func_evaluations_single_direct_search:
-    #             self.total_func_eval_track += self.temporary_func_eval_track
-    #             self.temporary_func_eval_track = 0
-    #
-    #         local_func_eval_track = self.total_func_eval_track + func_evaluations_single_direct_search
-    #         self.temporary_func_eval_track = func_evaluations_single_direct_search
-    #
-    #         terminate_reason = "Exceeded number of allowed function evaluations in total."
-    #         is_converged = False
-    #
-    #         if local_func_eval_track >= self.max_eval_total:
-    #             is_converged = True
-    #             # this is a bit confusing but here we set is_running to False if is is linked
-    #             if self.stop_opt_function is not None:
-    #                 self.stop_opt_function()
-    #         return [is_converged, terminate_reason]
-    #
-    #     else:
-    #         terminate_reason = "Exceeded number of allowed function evaluations in total."
-    #         is_converged = False
-    #         return [is_converged, terminate_reason]
-
     def check_simplex_criterion(self, sim: np.array) -> [bool, str]:
         """
         Check whether the simplex has converged
@@ -127,23 +95,6 @@ class StoppingCriteria:
             is_converged = True
         return [is_converged, terminate_reason]
 
-    # def check_goal_reached(self, FoM: float) -> [bool, str]:
-    #     """
-    #     Check whether the FoM goal has been reached
-    #
-    #     :param float FoM: current FoM
-    #     :return bool is_converged: True if the stopping criterion is fulfilled
-    #     :return str terminate_reason: reason for the terminatiom
-    #     """
-    #     terminate_reason = "FoM goal reached."
-    #     is_converged = False
-    #     if FoM <= self.FoM_goal:
-    #         is_converged = True
-    #         # this is a bit confusing but here we set is_running to False if is is linked
-    #         if self.stop_opt_function is not None:
-    #             self.stop_opt_function()
-    #     return [is_converged, terminate_reason]
-
     def reset_direct_search_start_time(self) -> None:
         """
         Reset the direct_search_start_time for a new SI
@@ -165,20 +116,40 @@ class StoppingCriteria:
             is_converged = True
         return [is_converged, terminate_reason]
 
-    # def check_total_time_out(self) -> [bool, str]:
-    #     """
-    #     Check whether the optimization has been running for too long.
-    #
-    #     :return bool is_converged: True if the stopping criterion is fulfilled
-    #     :return str terminate_reason: reason for the terminatiom
-    #     """
-    #     end_time = datetime.now()
-    #     minutes_diff = (end_time - self.total_start_time).total_seconds() / 60.0
-    #     terminate_reason = "Total optimization time exceeded limit."
-    #     is_converged = False
-    #     if minutes_diff >= self.total_time_lim:
-    #         is_converged = True
-    #         # this is a bit confusing but here we set is_running to False if is is linked
-    #         if self.stop_opt_function is not None:
-    #             self.stop_opt_function()
-    #     return [is_converged, terminate_reason]
+    def reset_curr_FoM_track_for_new_SI(self) -> None:
+        """
+        Reset the current FoM track list for a new SI
+        """
+        self.curr_FoM_track = []
+
+    def add_to_FoM_track(self, FoM) -> None:
+        """
+        Add new entry to current FoM track list
+        """
+        if self.change_based_stop["cbs_funct_evals"] > 1:
+            self.curr_FoM_track.append(FoM)
+
+    def _check_cbs_stopping_crit(self) -> None:
+        """
+        Check the change-based stopping criterion
+        """
+        self.terminate_reason = "Change-based stopping criterion reached."
+        cbs_funct_evals = self.change_based_stop["cbs_funct_evals"]
+        cbs_change = self.change_based_stop["cbs_change"]
+        if len(self.curr_FoM_track) >= cbs_funct_evals:
+            m, b = np.polyfit(range(cbs_funct_evals), self.curr_FoM_track[-cbs_funct_evals:], 1)
+            current_change = abs(m * cbs_funct_evals)
+            # check if the trend of changes is smaller than defined
+            if (current_change < cbs_change):
+                self.is_converged = True
+
+    def check_advanced_stopping_criteria(self) -> None:
+        """
+        Check the advanced stopping criteria if they are defined
+        """
+        try:
+            if self.change_based_stop["cbs_funct_evals"] > 1:
+                self._check_cbs_stopping_crit()
+        except:
+            message = "Checking change-based stop failed! cbs_funct_evals not properly defined!"
+            self.logger.error(message)
